@@ -21,33 +21,31 @@
 
 #include "nvme-ioctl.h"
 
-int nvme_subsystem_reset(int fd)
+void nvme_verify_chr(int fd)
 {
 	static struct stat nvme_stat;
 	int err = fstat(fd, &nvme_stat);
 
-	if (err < 0)
-		return err;
-	if (!S_ISCHR(nvme_stat.st_mode)) {
-		fprintf(stderr,
-			"Error: requesting subsystem reset on non-controller handle\n");
-		exit(ENOTBLK);
+	if (err < 0) {
+		perror("fstat");
+		exit(err);
 	}
-	return ioctl(fd, NVME_IOCTL_SUBSYS_RESET);
-}
-
-int nvme_reset_controller(int fd)
-{
-	static struct stat nvme_stat;
-	int err = fstat(fd, &nvme_stat);
-
-	if (err < 0)
-		return err;
 	if (!S_ISCHR(nvme_stat.st_mode)) {
 		fprintf(stderr,
 			"Error: requesting reset on non-controller handle\n");
 		exit(ENOTBLK);
 	}
+}
+
+int nvme_subsystem_reset(int fd)
+{
+	nvme_verify_chr(fd);
+	return ioctl(fd, NVME_IOCTL_SUBSYS_RESET);
+}
+
+int nvme_reset_controller(int fd)
+{
+	nvme_verify_chr(fd);
 	return ioctl(fd, NVME_IOCTL_RESET);
 }
 
@@ -337,61 +335,72 @@ int nvme_identify_ctrl(int fd, void *data)
 
 int nvme_identify_ns(int fd, __u32 nsid, bool present, void *data)
 {
-	return nvme_identify(fd, nsid, present ? 0x11 : 0, data);
+	int cns = present ? NVME_ID_CNS_NS_PRESENT : NVME_ID_CNS_NS;
+
+	return nvme_identify(fd, nsid, cns, data);
 }
 
 int nvme_identify_ns_list(int fd, __u32 nsid, bool all, void *data)
 {
-	return nvme_identify(fd, nsid, all ? 0x10 : 0x2, data);
+	int cns = all ? NVME_ID_CNS_NS_ACTIVE_LIST : NVME_ID_CNS_NS_PRESENT_LIST;
+
+	return nvme_identify(fd, nsid, cns, data);
 }
 
 int nvme_identify_ctrl_list(int fd, __u32 nsid, __u16 cntid, void *data)
 {
-	return nvme_identify(fd, nsid, (cntid << 16) | (nsid ? 0x13 : 0x12), data);
+	int cns = nsid ? NVME_ID_CNS_CTRL_NS_LIST : NVME_ID_CNS_CTRL_LIST;
+
+	return nvme_identify(fd, nsid, (cntid << 16) | cns, data);
 }
 
-int nvme_get_log(int fd, __u32 nsid, __u32 cdw10, __u32 data_len, void *data)
+int nvme_get_log(int fd, __u32 nsid, __u8 log_id, __u32 data_len, void *data)
 {
 	struct nvme_admin_cmd cmd = {
 		.opcode		= nvme_admin_get_log_page,
 		.nsid		= nsid,
 		.addr		= (__u64)(uintptr_t) data,
 		.data_len	= data_len,
-		.cdw10		= cdw10,
 	};
+	__u32 numd = (data_len >> 2) - 1;
+	__le16 numdu, numdl;
+
+	numdu = htole16((numd & 0xFFFF0000) >> 16);
+	numdl = htole16(numd & 0x0000FFFF);
+	cmd.cdw10 = log_id | (numdl << 16);
+	cmd.cdw11 = numdu;
+
 	return nvme_submit_admin_passthru(fd, &cmd);
 }
 
-int nvme_log(int fd, __u32 nsid, __u8 log_id, __u32 data_len, void *data)
-{
-	__u32 cdw10 = log_id | ((data_len >> 2) - 1) << 16;
-
-	return nvme_get_log(fd, nsid, cdw10, data_len, data);
-}
-
 int nvme_fw_log(int fd, struct nvme_firmware_log_page *fw_log)
 {
-	return nvme_log(fd, 0xffffffff, 3, sizeof(*fw_log), fw_log);
+	return nvme_get_log(fd, 0xffffffff, NVME_LOG_FW_SLOT, sizeof(*fw_log), fw_log);
 }
 
 int nvme_error_log(int fd, __u32 nsid, int entries,
 		   struct nvme_error_log_page *err_log)
 {
-	return nvme_log(fd, nsid, 1, entries * sizeof(*err_log), err_log);
+	return nvme_get_log(fd, nsid, NVME_LOG_ERROR, entries * sizeof(*err_log), err_log);
 }
 
 int nvme_smart_log(int fd, __u32 nsid, struct nvme_smart_log *smart_log)
 {
-	return nvme_log(fd, nsid, 2, sizeof(*smart_log), smart_log);
+	return nvme_get_log(fd, nsid, NVME_LOG_SMART, sizeof(*smart_log), smart_log);
 }
 
 int nvme_intel_smart_log(int fd, __u32 nsid,
 			 struct nvme_additional_smart_log *intel_smart_log)
 {
-	return nvme_log(fd, nsid, 0xca, sizeof(*intel_smart_log),
+	return nvme_get_log(fd, nsid, 0xca, sizeof(*intel_smart_log),
 			intel_smart_log);
 }
 
+int nvme_discovery_log(int fd, struct nvmf_disc_rsp_page_hdr *log, __u32 size)
+{
+	return nvme_get_log(fd, 0, NVME_LOG_DISC, size, log);
+}
+
 int nvme_feature(int fd, __u8 opcode, __u32 nsid, __u32 cdw10, __u32 cdw11,
 		 __u32 data_len, void *data, __u32 *result)
 {
@@ -487,6 +496,7 @@ int nvme_ns_attachment(int fd, __u32 nsid, __u16 num_ctrls, __u16 *ctrlist,
 					(struct nvme_controller_list *)buf;
 	struct nvme_admin_cmd cmd = {
 		.opcode		= nvme_admin_ns_attach,
+		.nsid		= nsid,
 		.addr		= (__u64)(uintptr_t) cntlist,
 		.cdw10		= attach ? 0 : 1,
 		.data_len	= 0x1000,
@@ -557,6 +567,7 @@ int nvme_sec_recv(int fd, __u32 nsid, __u8 nssf, __u16 spsp,
 {
 	struct nvme_admin_cmd cmd = {
 		.opcode		= nvme_admin_security_recv,
+		.nsid		= nsid,
 		.cdw10		= secp << 24 | spsp << 8 | nssf,
 		.cdw11		= al,
 		.addr		= (__u64)(uintptr_t) data,
