@@ -1,8 +1,10 @@
 #include <endian.h>
 #include <inttypes.h>
 #include <stdio.h>
+#include <string.h>
 
 #include "nvme-print.h"
+#include "json.h"
 
 static long double int128_to_double(__u8 *data)
 {
@@ -539,7 +541,7 @@ static void print_ps_power_and_scale(__le16 ctr_power, __u8 scale)
 	}
 }
 
-static void show_nvme_id_ctrl_power(struct nvme_id_ctrl *ctrl, unsigned int mode)
+static void show_nvme_id_ctrl_power(struct nvme_id_ctrl *ctrl)
 {
 	int i;
 
@@ -563,24 +565,24 @@ static void show_nvme_id_ctrl_power(struct nvme_id_ctrl *ctrl, unsigned int mode
 			ctrl->psd[i].read_tput, ctrl->psd[i].read_lat,
 			ctrl->psd[i].write_tput, ctrl->psd[i].write_lat);
 		print_ps_power_and_scale(ctrl->psd[i].idle_power,
-					 ctrl->psd[i].idle_scale);
+					 POWER_SCALE(ctrl->psd[i].idle_scale));
 		printf(" active_power:");
 		print_ps_power_and_scale(ctrl->psd[i].active_power,
-					 ctrl->psd[i].active_work_scale);
+					 POWER_SCALE(ctrl->psd[i].active_work_scale));
 		printf("\n");
 
 	}
 }
 
-void show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
+void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*vendor_show)(__u8 *vs))
 {
 	int human = mode & HUMAN, vs = mode & VS;
 
 	printf("vid     : %#x\n", le16toh(ctrl->vid));
 	printf("ssvid   : %#x\n", le16toh(ctrl->ssvid));
-	printf("sn      : %-20.20s\n", ctrl->sn);
-	printf("mn      : %-20.20s\n", ctrl->mn);
-	printf("fr      : %-.8s\n", ctrl->fr);
+	printf("sn      : %-.*s\n", (int)sizeof(ctrl->sn), ctrl->sn);
+	printf("mn      : %-.*s\n", (int)sizeof(ctrl->mn), ctrl->mn);
+	printf("fr      : %-.*s\n", (int)sizeof(ctrl->fr), ctrl->fr);
 	printf("rab     : %d\n", ctrl->rab);
 	printf("ieee    : %02x%02x%02x\n",
 		ctrl->ieee[2], ctrl->ieee[1], ctrl->ieee[0]);
@@ -653,13 +655,20 @@ void show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
 	if (human)
 		show_nvme_id_ctrl_sgls(ctrl->sgls);
 
-	show_nvme_id_ctrl_power(ctrl, mode);
-	if (vs) {
+	show_nvme_id_ctrl_power(ctrl);
+	if (vendor_show)
+		vendor_show(ctrl->vs);
+	else if (vs) {
 		printf("vs[]:\n");
 		d(ctrl->vs, sizeof(ctrl->vs), 16, 1);
 	}
 }
 
+void show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
+{
+	__show_nvme_id_ctrl(ctrl, mode, NULL);
+}
+
 void show_error_log(struct nvme_error_log_page *err_log, int entries, const char *devname)
 {
 	int i;
@@ -676,7 +685,7 @@ void show_error_log(struct nvme_error_log_page *err_log, int entries, const char
 		printf("status_field : %#x\n", err_log[i].status_field);
 		printf("parm_err_loc : %#x\n", err_log[i].parm_error_location);
 		printf("lba          : %#"PRIx64"\n",(uint64_t)le64toh(err_log[i].lba));
-		printf("nsid         : %d\n", err_log[i].nsid);
+		printf("nsid         : %#x\n", err_log[i].nsid);
 		printf("vs           : %d\n", err_log[i].vs);
 		printf(".................\n");
 	}
@@ -870,42 +879,60 @@ char* nvme_select_to_string(int sel)
 char *nvme_status_to_string(__u32 status)
 {
 	switch (status & 0x3ff) {
-	case NVME_SC_SUCCESS:		return "SUCCESS";
-	case NVME_SC_INVALID_OPCODE:	return "INVALID_OPCODE";
-	case NVME_SC_INVALID_FIELD:	return "INVALID_FIELD";
-	case NVME_SC_CMDID_CONFLICT:	return "CMDID_CONFLICT";
-	case NVME_SC_DATA_XFER_ERROR:	return "DATA_XFER_ERROR";
-	case NVME_SC_POWER_LOSS:	return "POWER_LOSS";
-	case NVME_SC_INTERNAL:		return "INTERNAL";
-	case NVME_SC_ABORT_REQ:		return "ABORT_REQ";
-	case NVME_SC_ABORT_QUEUE:	return "ABORT_QUEUE";
-	case NVME_SC_FUSED_FAIL:	return "FUSED_FAIL";
-	case NVME_SC_FUSED_MISSING:	return "FUSED_MISSING";
-	case NVME_SC_INVALID_NS:	return "INVALID_NS";
-	case NVME_SC_CMD_SEQ_ERROR:	return "CMD_SEQ_ERROR";
-	case NVME_SC_LBA_RANGE:		return "LBA_RANGE";
-	case NVME_SC_CAP_EXCEEDED:	return "CAP_EXCEEDED";
-	case NVME_SC_NS_NOT_READY:	return "NS_NOT_READY";
-	case NVME_SC_CQ_INVALID:	return "CQ_INVALID";
-	case NVME_SC_QID_INVALID:	return "QID_INVALID";
-	case NVME_SC_QUEUE_SIZE:	return "QUEUE_SIZE";
-	case NVME_SC_ABORT_LIMIT:	return "ABORT_LIMIT";
-	case NVME_SC_ABORT_MISSING:	return "ABORT_MISSING";
-	case NVME_SC_ASYNC_LIMIT:	return "ASYNC_LIMIT";
-	case NVME_SC_FIRMWARE_SLOT:	return "FIRMWARE_SLOT";
-	case NVME_SC_FIRMWARE_IMAGE:	return "FIRMWARE_IMAGE";
-	case NVME_SC_INVALID_VECTOR:	return "INVALID_VECTOR";
-	case NVME_SC_INVALID_LOG_PAGE:	return "INVALID_LOG_PAGE";
-	case NVME_SC_INVALID_FORMAT:	return "INVALID_FORMAT";
-	case NVME_SC_BAD_ATTRIBUTES:	return "BAD_ATTRIBUTES";
-	case NVME_SC_WRITE_FAULT:	return "WRITE_FAULT";
-	case NVME_SC_READ_ERROR:	return "READ_ERROR";
-	case NVME_SC_GUARD_CHECK:	return "GUARD_CHECK";
-	case NVME_SC_APPTAG_CHECK:	return "APPTAG_CHECK";
-	case NVME_SC_REFTAG_CHECK:	return "REFTAG_CHECK";
-	case NVME_SC_COMPARE_FAILED:	return "COMPARE_FAILED";
-	case NVME_SC_ACCESS_DENIED:	return "ACCESS_DENIED";
-	default:			return "Unknown";
+	case NVME_SC_SUCCESS:			return "SUCCESS";
+	case NVME_SC_INVALID_OPCODE:		return "INVALID_OPCODE";
+	case NVME_SC_INVALID_FIELD:		return "INVALID_FIELD";
+	case NVME_SC_CMDID_CONFLICT:		return "CMDID_CONFLICT";
+	case NVME_SC_DATA_XFER_ERROR:		return "DATA_XFER_ERROR";
+	case NVME_SC_POWER_LOSS:		return "POWER_LOSS";
+	case NVME_SC_INTERNAL:			return "INTERNAL";
+	case NVME_SC_ABORT_REQ:			return "ABORT_REQ";
+	case NVME_SC_ABORT_QUEUE:		return "ABORT_QUEUE";
+	case NVME_SC_FUSED_FAIL:		return "FUSED_FAIL";
+	case NVME_SC_FUSED_MISSING:		return "FUSED_MISSING";
+	case NVME_SC_INVALID_NS:		return "INVALID_NS";
+	case NVME_SC_CMD_SEQ_ERROR:		return "CMD_SEQ_ERROR";
+	case NVME_SC_LBA_RANGE:			return "LBA_RANGE";
+	case NVME_SC_CAP_EXCEEDED:		return "CAP_EXCEEDED";
+	case NVME_SC_NS_NOT_READY:		return "NS_NOT_READY";
+	case NVME_SC_CQ_INVALID:		return "CQ_INVALID";
+	case NVME_SC_QID_INVALID:		return "QID_INVALID";
+	case NVME_SC_QUEUE_SIZE:		return "QUEUE_SIZE";
+	case NVME_SC_ABORT_LIMIT:		return "ABORT_LIMIT";
+	case NVME_SC_ABORT_MISSING:		return "ABORT_MISSING";
+	case NVME_SC_ASYNC_LIMIT:		return "ASYNC_LIMIT";
+	case NVME_SC_FIRMWARE_SLOT:		return "FIRMWARE_SLOT";
+	case NVME_SC_FIRMWARE_IMAGE:		return "FIRMWARE_IMAGE";
+	case NVME_SC_INVALID_VECTOR:		return "INVALID_VECTOR";
+	case NVME_SC_INVALID_LOG_PAGE:		return "INVALID_LOG_PAGE";
+	case NVME_SC_INVALID_FORMAT:		return "INVALID_FORMAT";
+	case NVME_SC_FW_NEEDS_CONV_RESET:	return "FW_NEEDS_CONVENTIONAL_RESET";
+	case NVME_SC_INVALID_QUEUE:		return "INVALID_QUEUE";
+	case NVME_SC_FEATURE_NOT_SAVEABLE:	return "FEATURE_NOT_SAVEABLE";
+	case NVME_SC_FEATURE_NOT_CHANGEABLE:	return "FEATURE_NOT_CHANGEABLE";
+	case NVME_SC_FEATURE_NOT_PER_NS:	return "FEATURE_NOT_PER_NS";
+	case NVME_SC_FW_NEEDS_SUBSYS_RESET:	return "FW_NEEDS_SUBSYSTEM_RESET";
+	case NVME_SC_FW_NEEDS_RESET:		return "FW_NEEDS_RESET";
+	case NVME_SC_FW_NEEDS_MAX_TIME:		return "FW_NEEDS_MAX_TIME_VIOLATION";
+	case NVME_SC_FW_ACIVATE_PROHIBITED:	return "FW_ACTIVATION_PROHIBITED";
+	case NVME_SC_OVERLAPPING_RANGE:		return "OVERLAPPING_RANGE";
+	case NVME_SC_NS_INSUFFICENT_CAP:	return "NS_INSUFFICIENT_CAPACITY";
+	case NVME_SC_NS_ID_UNAVAILABLE:		return "NS_ID_UNAVAILABLE";
+	case NVME_SC_NS_ALREADY_ATTACHED:	return "NS_ALREADY_ATTACHED";
+	case NVME_SC_NS_IS_PRIVATE:		return "NS_IS_PRIVATE";
+	case NVME_SC_NS_NOT_ATTACHED:		return "NS_NOT_ATTACHED";
+	case NVME_SC_THIN_PROV_NOT_SUPP:	return "THIN_PROVISIONING_NOT_SUPPORTED";
+	case NVME_SC_CTRL_LIST_INVALID:		return "CONTROLLER_LIST_INVALID";
+	case NVME_SC_BAD_ATTRIBUTES:		return "BAD_ATTRIBUTES";
+	case NVME_SC_WRITE_FAULT:		return "WRITE_FAULT";
+	case NVME_SC_READ_ERROR:		return "READ_ERROR";
+	case NVME_SC_GUARD_CHECK:		return "GUARD_CHECK";
+	case NVME_SC_APPTAG_CHECK:		return "APPTAG_CHECK";
+	case NVME_SC_REFTAG_CHECK:		return "REFTAG_CHECK";
+	case NVME_SC_COMPARE_FAILED:		return "COMPARE_FAILED";
+	case NVME_SC_ACCESS_DENIED:		return "ACCESS_DENIED";
+	case NVME_SC_UNWRITTEN_BLOCK:		return "UNWRITTEN_BLOCK";
+	default:				return "Unknown";
 	}
 }
 
@@ -1012,7 +1039,11 @@ void nvme_feature_show_fields(__u32 fid, unsigned int result, unsigned char *buf
 		printf("\tHigh Priority Weight   (HPW): %u\n", ((result & 0xff000000) >> 24) + 1);
 		printf("\tMedium Priority Weight (MPW): %u\n", ((result & 0x00ff0000) >> 16) + 1);
 		printf("\tLow Priority Weight    (LPW): %u\n", ((result & 0x0000ff00) >> 8) + 1);
-		printf("\tArbitration Burst       (AB): %u\n",  1 << (result & 0x00000007));
+		printf("\tArbitration Burst       (AB): ");
+		if ((result & 0x00000007) == 7)
+			printf("No limit\n");
+		else
+			printf("%u\n",  1 << (result & 0x00000007));
 		break;
 	case NVME_FEAT_POWER_MGMT:
 		field = (result & 0x000000E0) >> 5;
@@ -1086,3 +1117,281 @@ void nvme_feature_show_fields(__u32 fid, unsigned int result, unsigned char *buf
 		break;
 	}
 }
+
+void json_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
+{
+	char nguid_buf[2 * sizeof(ns->nguid) + 1], eui64_buf[2 * sizeof(ns->eui64) + 1];
+	char *nguid = nguid_buf, *eui64 = eui64_buf;
+	struct json_object *root;
+	struct json_array *lbafs;
+	int i;
+
+	long double nvmcap = int128_to_double(ns->nvmcap);
+
+	root = json_create_object();
+
+	json_object_add_value_int(root, "nsze", ns->nsze);
+	json_object_add_value_int(root, "ncap", ns->ncap);
+	json_object_add_value_int(root, "nuse", ns->nuse);
+	json_object_add_value_int(root, "nsfeat", ns->nsfeat);
+	json_object_add_value_int(root, "nlbaf", ns->nlbaf);
+	json_object_add_value_int(root, "flbas", ns->flbas);
+	json_object_add_value_int(root, "mc", ns->mc);
+	json_object_add_value_int(root, "dpc", ns->dpc);
+	json_object_add_value_int(root, "dps", ns->dps);
+	json_object_add_value_int(root, "nmic", ns->nmic);
+	json_object_add_value_int(root, "rescap", ns->rescap);
+	json_object_add_value_int(root, "fpi", ns->fpi);
+	json_object_add_value_int(root, "nawun", ns->nawun);
+	json_object_add_value_int(root, "nawupf", ns->nawupf);
+	json_object_add_value_int(root, "nacwu", ns->nacwu);
+	json_object_add_value_int(root, "nabsn", ns->nabsn);
+	json_object_add_value_int(root, "nabo", ns->nabo);
+	json_object_add_value_int(root, "nabspf", ns->nabspf);
+	json_object_add_value_float(root, "nvmcap", nvmcap);
+
+	memset(eui64, 0, sizeof(eui64_buf));
+	for (i = 0; i < sizeof(ns->eui64); i++)
+		eui64 += sprintf(eui64, "%02x", ns->eui64[i]);
+
+	memset(nguid, 0, sizeof(nguid_buf));
+	for (i = 0; i < sizeof(ns->nguid); i++)
+		nguid += sprintf(nguid, "%02x", ns->nguid[i]);
+
+	json_object_add_value_string(root, "eui64", eui64_buf);
+	json_object_add_value_string(root, "nguid", nguid_buf);
+
+	lbafs = json_create_array();
+	json_object_add_value_array(root, "lbafs", lbafs);
+
+	for (i = 0; i <= ns->nlbaf; i++) {
+		struct json_object *lbaf = json_create_object();
+
+		json_object_add_value_int(lbaf, "ms", ns->lbaf[i].ms);
+		json_object_add_value_int(lbaf, "ds", ns->lbaf[i].ds);
+		json_object_add_value_int(lbaf, "rp", ns->lbaf[i].rp);
+
+		json_array_add_value_object(lbafs, lbaf);
+	}
+
+	json_print_object(root, NULL);
+	printf("\n");
+}
+
+void json_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
+{
+	struct json_object *root;
+	struct json_array *psds;
+
+	long double tnvmcap = int128_to_double(ctrl->tnvmcap);
+	long double unvmcap = int128_to_double(ctrl->unvmcap);
+
+	char sn[sizeof(ctrl->sn) + 1], mn[sizeof(ctrl->mn) + 1], fr[sizeof(ctrl->fr) + 1];
+	__u32 ieee = ctrl->ieee[2] << 16 | ctrl->ieee[1] << 8 | ctrl->ieee[0];
+
+	int i;
+
+	sprintf(sn, "%-.*s\n", (int)sizeof(ctrl->sn), ctrl->sn);
+	sprintf(mn, "%-.*s\n", (int)sizeof(ctrl->mn), ctrl->mn);
+	sprintf(fr, "%-.*s\n", (int)sizeof(ctrl->fr), ctrl->fr);
+
+	root = json_create_object();
+
+	json_object_add_value_int(root, "vid", ctrl->vid);
+	json_object_add_value_int(root, "ssvid", ctrl->ssvid);
+	json_object_add_value_int(root, "rab", ctrl->rab);
+	json_object_add_value_int(root, "ieee", ieee);
+	json_object_add_value_int(root, "cmic", ctrl->cmic);
+	json_object_add_value_int(root, "mdts", ctrl->mdts);
+	json_object_add_value_int(root, "cntlid", ctrl->cntlid);
+	json_object_add_value_int(root, "ver", ctrl->ver);
+	json_object_add_value_int(root, "rtd3r", ctrl->rtd3r);
+	json_object_add_value_int(root, "rtd3e", ctrl->rtd3e);
+	json_object_add_value_int(root, "oaes", ctrl->oaes);
+	json_object_add_value_int(root, "oacs", ctrl->oacs);
+	json_object_add_value_int(root, "acl", ctrl->acl);
+	json_object_add_value_int(root, "aerl", ctrl->aerl);
+	json_object_add_value_int(root, "frmw", ctrl->frmw);
+	json_object_add_value_int(root, "lpa", ctrl->lpa);
+	json_object_add_value_int(root, "elpe", ctrl->elpe);
+	json_object_add_value_int(root, "npss", ctrl->npss);
+	json_object_add_value_int(root, "avscc", ctrl->avscc);
+	json_object_add_value_int(root, "apsta", ctrl->apsta);
+	json_object_add_value_int(root, "wctemp", ctrl->wctemp);
+	json_object_add_value_int(root, "cctemp", ctrl->cctemp);
+	json_object_add_value_int(root, "mtfa", ctrl->mtfa);
+	json_object_add_value_int(root, "hmpre", ctrl->hmpre);
+	json_object_add_value_int(root, "hmmin", ctrl->hmmin);
+	json_object_add_value_float(root, "tnvmcap", tnvmcap);
+	json_object_add_value_float(root, "unvmcap", unvmcap);
+	json_object_add_value_int(root, "rpmbs", ctrl->rpmbs);
+	json_object_add_value_int(root, "sqes", ctrl->sqes);
+	json_object_add_value_int(root, "cqes", ctrl->cqes);
+	json_object_add_value_int(root, "nn", ctrl->nn);
+	json_object_add_value_int(root, "oncs", ctrl->oncs);
+	json_object_add_value_int(root, "fuses", ctrl->fuses);
+	json_object_add_value_int(root, "fna", ctrl->fna);
+	json_object_add_value_int(root, "vwc", ctrl->vwc);
+	json_object_add_value_int(root, "awun", ctrl->awun);
+	json_object_add_value_int(root, "awupf", ctrl->awupf);
+	json_object_add_value_int(root, "nvscc", ctrl->nvscc);
+	json_object_add_value_int(root, "acwu", ctrl->acwu);
+	json_object_add_value_int(root, "sgls", ctrl->sgls);
+
+	psds = json_create_array();
+	json_object_add_value_array(root, "psds", psds);
+
+	for (i = 0; i <= ctrl->npss; i++) {
+		struct json_object *psd = json_create_object();
+
+		json_object_add_value_int(psd, "max_power", ctrl->psd[i].max_power);
+		json_object_add_value_int(psd, "flags", ctrl->psd[i].flags);
+		json_object_add_value_int(psd, "entry_lat", ctrl->psd[i].entry_lat);
+		json_object_add_value_int(psd, "exit_lat", ctrl->psd[i].exit_lat);
+		json_object_add_value_int(psd, "read_tput", ctrl->psd[i].read_tput);
+		json_object_add_value_int(psd, "read_lat", ctrl->psd[i].read_lat);
+		json_object_add_value_int(psd, "write_tput", ctrl->psd[i].write_tput);
+		json_object_add_value_int(psd, "write_lat", ctrl->psd[i].write_lat);
+		json_object_add_value_int(psd, "idle_power", ctrl->psd[i].idle_power);
+		json_object_add_value_int(psd, "idle_scale", ctrl->psd[i].idle_scale);
+		json_object_add_value_int(psd, "active_power", ctrl->psd[i].active_power);
+		json_object_add_value_int(psd, "active_work_scale", ctrl->psd[i].active_work_scale);
+
+		json_array_add_value_object(psds, psd);
+	}
+
+	json_print_object(root, NULL);
+	printf("\n");
+}
+
+void json_error_log(struct nvme_error_log_page *err_log, int entries, const char *devname)
+{
+	struct json_object *root;
+	struct json_array *errors;
+
+	int i;
+
+	root = json_create_object();
+
+	errors = json_create_array();
+	json_object_add_value_array(root, "errors", errors);
+
+	for (i = 0; i < entries; i++) {
+		struct json_object *error = json_create_object();
+
+		json_object_add_value_int(error, "error_count", err_log[i].error_count);
+		json_object_add_value_int(error, "sqid", err_log[i].sqid);
+		json_object_add_value_int(error, "cmdid", err_log[i].cmdid);
+		json_object_add_value_int(error, "status_field", err_log[i].status_field);
+		json_object_add_value_int(error, "parm_error_location", err_log[i].parm_error_location);
+		json_object_add_value_int(error, "lba", err_log[i].lba);
+		json_object_add_value_int(error, "nsid", err_log[i].nsid);
+		json_object_add_value_int(error, "vs", err_log[i].vs);
+
+		json_array_add_value_object(errors, error);
+	}
+
+	json_print_object(root, NULL);
+	printf("\n");
+}
+
+void json_nvme_resv_report(struct nvme_reservation_status *status)
+{
+	struct json_object *root;
+	struct json_array *rcs;
+	int i, regctl;
+
+	regctl = status->regctl[0] | (status->regctl[1] << 8);
+
+	root = json_create_object();
+
+	json_object_add_value_int(root, "gen", le32toh(status->gen));
+	json_object_add_value_int(root, "regctl", regctl);
+	json_object_add_value_int(root, "rtype", status->rtype);
+	json_object_add_value_int(root, "ptpls", status->ptpls);
+
+	rcs = json_create_array();
+	json_object_add_value_array(root, "regctls", rcs);
+
+	for (i = 0; i < regctl; i++) {
+		struct json_object *rc = json_create_object();
+
+		json_object_add_value_int(rc, "cntlid", le16toh(status->regctl_ds[i].cntlid));
+		json_object_add_value_int(rc, "rcsts", status->regctl_ds[i].rcsts);
+		json_object_add_value_int(rc, "hostid", (uint64_t)le64toh(status->regctl_ds[i].hostid));
+		json_object_add_value_int(rc, "rkey", (uint64_t)le64toh(status->regctl_ds[i].rkey));
+
+		json_array_add_value_object(rcs, rc);
+	}
+
+	json_print_object(root, NULL);
+	printf("\n");
+}
+
+void json_fw_log(struct nvme_firmware_log_page *fw_log, const char *devname)
+{
+	struct json_object *root;
+	struct json_array *frs;
+	int i;
+
+	root = json_create_object();
+
+	json_object_add_value_int(root, "afi", fw_log->afi);
+
+	frs = json_create_array();
+	json_object_add_value_array(root, "frs", frs);
+
+	for (i = 0; i < 7; i++) {
+		struct json_object *fr = json_create_object();
+
+		json_object_add_value_int(fr, "frs", fw_log->frs[i]);
+
+		json_array_add_value_object(frs, fr);
+
+	}
+
+	json_print_object(root, NULL);
+	printf("\n");
+}
+
+void json_smart_log(struct nvme_smart_log *smart, unsigned int nsid, const char *devname)
+{
+	struct json_object *root;
+
+	unsigned int temperature = ((smart->temperature[1] << 8) |
+		smart->temperature[0]);
+
+	long double data_units_read = int128_to_double(smart->data_units_read);
+	long double data_units_written = int128_to_double(smart->data_units_written);
+	long double host_read_commands = int128_to_double(smart->host_reads);
+	long double host_write_commands = int128_to_double(smart->host_writes);
+	long double controller_busy_time = int128_to_double(smart->ctrl_busy_time);
+	long double power_cycles = int128_to_double(smart->power_cycles);
+	long double power_on_hours = int128_to_double(smart->power_on_hours);
+	long double unsafe_shutdowns = int128_to_double(smart->unsafe_shutdowns);
+	long double media_errors = int128_to_double(smart->media_errors);
+	long double num_err_log_entries = int128_to_double(smart->num_err_log_entries);
+
+	root = json_create_object();
+
+	json_object_add_value_int(root, "critical_warning", smart->critical_warning);
+	json_object_add_value_int(root, "temperature", temperature);
+	json_object_add_value_int(root, "avail_spare", smart->avail_spare);
+	json_object_add_value_int(root, "spare_thresh", smart->spare_thresh);
+	json_object_add_value_int(root, "percent_used", smart->percent_used);
+	json_object_add_value_float(root, "data_units_read", data_units_read);
+	json_object_add_value_float(root, "data_units_written", data_units_written);
+	json_object_add_value_float(root, "host_read_commands", host_read_commands);
+	json_object_add_value_float(root, "host_write_commands", host_write_commands);
+	json_object_add_value_float(root, "controller_busy_time", controller_busy_time);
+	json_object_add_value_float(root, "power_cycles", power_cycles);
+	json_object_add_value_float(root, "power_on_hours", power_on_hours);
+	json_object_add_value_float(root, "unsafe_shutdowns", unsafe_shutdowns);
+	json_object_add_value_float(root, "media_errors", media_errors);
+	json_object_add_value_float(root, "num_err_log_entries", num_err_log_entries);
+	json_object_add_value_int(root, "warning_temp_time", smart->warning_temp_time);
+	json_object_add_value_int(root, "critical_comp_time", smart->critical_comp_time);
+
+	json_print_object(root, NULL);
+	printf("\n");
+}
