@@ -1,7 +1,7 @@
-#ifndef COMMON_H
-#define COMMON_H
+#ifndef NVME_PRINT_H
+#define NVME_PRINT_H
 
-#include "linux/nvme.h"
+#include "nvme.h"
 
 enum {
 	TERSE = 0x1u,	// only show a few useful fields
@@ -13,6 +13,7 @@ enum {
 void d(unsigned char *buf, int len, int width, int group);
 void d_raw(unsigned char *buf, unsigned len);
 
+void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*vendor_show)(__u8 *vs));
 void show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode);
 void show_nvme_id_ns(struct nvme_id_ns *ns, unsigned int flags);
 void show_nvme_resv_report(struct nvme_reservation_status *status);
@@ -27,4 +28,12 @@ char *nvme_status_to_string(__u32 status);
 char *nvme_select_to_string(int sel);
 char *nvme_feature_to_string(int feature);
 
+void json_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode);
+void json_nvme_id_ns(struct nvme_id_ns *ns, unsigned int flags);
+void json_nvme_resv_report(struct nvme_reservation_status *status);
+void json_error_log(struct nvme_error_log_page *err_log, int entries, const char *devname);
+void json_smart_log(struct nvme_smart_log *smart, unsigned int nsid, const char *devname);
+void json_fw_log(struct nvme_firmware_log_page *fw_log, const char *devname);
+
+
 #endif
